from http import HTTPStatus
from urllib.parse import urlencode

import flask.testing
import pytest
from faker import Faker

from .conftest import use_provider_config

faker = Faker()


@pytest.mark.parametrize("method", ["GET", "POST"])
@use_provider_config(require_client_registration=True)
def test_invalid_client(client: flask.testing.FlaskClient, method: str):
    """
    Respond with 400 and error description when:

    * client_id query parameter is missing
    * client unknown
    * redirect_uri does not match the URI that was registered
    """

    query = urlencode({
        "redirect_uri": "foo",
        "response_type": "code",
    })
    response = client.open(f"/oauth2/authorize?{query}", method=method)
    assert response.status_code == 400
    assert "Error: invalid_client" in response.text
    # TODO improve error message: tell user that is missing
    assert "Invalid client_id query parameter" in response.text

    query = urlencode({
        "client_id": "UNKNOWN",
        "redirect_uri": "foo",
        "response_type": "code",
    })
    response = client.open(f"/oauth2/authorize?{query}", method=method)
    assert response.status_code == 400
    assert "Error: invalid_client" in response.text
    # TODO improve error message
    assert "Invalid client_id query parameter" in response.text

    redirect_uris = [faker.uri(schemes=["https"])]
    response = client.post(
        "/oauth2/clients",
        json={
            "redirect_uris": redirect_uris,
        },
    )
    assert response.status_code == HTTPStatus.CREATED
    oidc_client = response.json
    assert oidc_client

    query = urlencode({
        "client_id": oidc_client["client_id"],
        "redirect_uri": "foo",
        "response_type": "code",
    })
    response = client.open(f"/oauth2/authorize?{query}", method=method)
    assert response.status_code == 400
    assert "Error: invalid_client" in response.text
    assert "Redirect URI foo is not supported by client." in response.text


def test_missing_sub_parameter(client: flask.testing.FlaskClient):
    query = urlencode({
        "client_id": str(faker.uuid4()),
        "redirect_uri": faker.uri(schemes=["https"]),
        "response_type": "code",
    })
    response = client.post(f"/oauth2/authorize?{query}")
    assert "The field is missing" in response.text



#!/usr/bin/env -S uv run

import os
import subprocess
import sys
from pathlib import Path

import toml


class UsageError(Exception):
    def __init__(self, msg: str) -> None:
        super().__init__(msg)
        self.msg = msg


def main():
    """Check that the git ref can be published.

    1. Check that the tag name matches the version in `pyproject.toml`
    2. Check that the commit message is `release: vX.Y.Z`
    """
    pyproject_data = toml.load(Path("pyproject.toml"))
    version = pyproject_data["project"]["version"]
    ref_prefix = "refs/tags/v"
    github_ref = os.getenv("GITHUB_REF")
    if not github_ref:
        raise UsageError("GITHUB_REF environment variable not set")
    if not github_ref.startswith(ref_prefix):
        raise UsageError(f"GITHUB_REF environment does not start with {ref_prefix}")

    tag_version = github_ref.removeprefix(ref_prefix)

    if tag_version != version:
        raise UsageError(
            f"Tagged version `{tag_version}` does not match version `{version}` from pyproject.toml "
        )

    commit_subject = subprocess.run(
        ["git", "log", "-1", "--pretty=%s"],
        stdout=subprocess.PIPE,
        text=True,
        check=True,
    ).stdout.strip()

    expected_subject = f"release: v{version}"
    if commit_subject != expected_subject:
        raise UsageError(
            "Invalid commit message for release publish.\n"
            f"Expected subject `{expected_subject}`, got `{commit_subject}`"
        )


if __name__ == "__main__":
    try:
        main()
    except UsageError as e:
        print(f"ERROR: {e.msg}")  # noqa: T201
        sys.exit(1)
